<?php

/**
 * Copy file to cloud Background Process
 *
 * @link       https://themeforest.net/user/nouthemes/portfolio
 * @since      1.0.22
 *
 * @package    Leopard_Wordpress_Offload_Media
 * @subpackage Leopard_Wordpress_Offload_Media/includes
 */

class Leopard_Wordpress_Offload_Media_Copy_To_Cloud_Process {

	/**
	 * Initiate new background process.
	 */
	public function __construct() {
		add_action( 'nou_leopard_offload_media_cronjob_copy_attachments_to_cloud', [ $this, 'task' ] );
	}

	public function unschedule(){
		try {
			if ( function_exists( 'as_next_scheduled_action' ) ) {
				leopard_wordpress_offload_media_unschedule_action( 'nou_leopard_offload_media_cronjob_copy_attachments_to_cloud' );
			}
		} catch (\Throwable $th) {}
		return false;
	}

	/**
	 * Complete
	 */
	public function complete() {
		
		try {
			if ( function_exists( 'as_next_scheduled_action' ) ) {
				leopard_wordpress_offload_media_unschedule_action( 'nou_leopard_offload_media_cronjob_copy_attachments_to_cloud' );
			}
		} catch (\Throwable $th) {}

		update_option('nou_leopard_offload_media_page_scaned_attachments', 1);
		update_option('nou_leopard_offload_media_scaned_pages_attachments', 0);

		update_option('nou_leopard_offload_media_copyed_to_cloud_status', 2);
		update_option('nou_leopard_offload_media_step_scan_attachments', 3);

		update_option('nou_leopard_offload_media_action', '');

		try{
			wp_mail( 
				get_option('admin_email'), 
				esc_html__('Leopard Offload Media Synchronize', 'leopard-wordpress-offload-media'), 
				esc_html__('Leopard Offload Media Copy all files from server to bucket: process has been completed.', 'leopard-wordpress-offload-media') 
			);
		} catch (Exception $e){
			error_log('wp_mail send failed.');
		}
	}

	public function clearScheduled()
	{
		try {
			if ( function_exists( 'as_next_scheduled_action' ) ) {
				leopard_wordpress_offload_media_unschedule_action( 'nou_leopard_offload_media_cronjob_scan_attachments' );
			}
		} catch (\Throwable $th) {}

		try {
			if ( function_exists( 'as_next_scheduled_action' ) ) {
				leopard_wordpress_offload_media_unschedule_action( 'nou_leopard_offload_media_cronjob_remove_files_from_server' );
			}
		} catch (\Throwable $th) {}

		try {
			if ( function_exists( 'as_next_scheduled_action' ) ) {
				leopard_wordpress_offload_media_unschedule_action( 'nou_leopard_offload_media_cronjob_download_files_from_bucket' );
			}
		} catch (\Throwable $th) {}

		try {
			if ( function_exists( 'as_next_scheduled_action' ) ) {
				leopard_wordpress_offload_media_unschedule_action( 'nou_leopard_offload_media_cronjob_remove_files_from_bucket' );
			}
		} catch (\Throwable $th) {}
	}

	/**
	 * Task
	 * 
	 * @return mixed
	 */
	public function task() {

		$this->clearScheduled();

		if(!leopard_wordpress_offload_media_cronjob_timed()){
			return false;
		}

		$status = get_option('nou_leopard_offload_media_copyed_to_cloud_status', 0);
		if($status == 0){
			return false;
		}

		$total_synced = get_option('nou_leopard_offload_media_copyed_to_cloud_data', []);
		if(!is_array($total_synced)){
			$total_synced = [];
		}

		$totalItems = get_option('nou_leopard_offload_media_scanned_attachments', []);
		if(count($totalItems) == 0){
			$this->unschedule();
		}
		
		$items = get_option('nou_leopard_offload_media_scanned_attachments_copy', []);
		if(count($items) == 0){
			update_option('nou_leopard_offload_media_scanned_attachments_copy', $totalItems);
		}
		$item = $items[0];

		update_option('nou_leopard_offload_media_step_scan_attachments', 2);

		error_log("Start copy {$item}");
		
		array_splice($items, 0, 1);
		
		update_option('nou_leopard_offload_media_scanned_attachments_copy', $items);

		$outputProgress = get_option('nou_leopard_offload_media_output_progress', []);
		$path = get_post_meta($item, '_wp_attached_file', true);
		$outputProgress[] = $path;
		update_option('nou_leopard_offload_media_output_progress', array_unique($outputProgress));
		
		try{
			leopard_offload_media_copy_to_s3_function($item);
		}catch(Exception $e){
			error_log($e->getMessage());
		}

		$total_synced[] = $item;
		update_option('nou_leopard_offload_media_copyed_to_cloud_data', array_unique($total_synced));
		
		if(count($items) == 0){
			$this->complete();
		}

		error_log("End copy {$item}");

		return false;
	}
}
?>